from nonebot import require
from nonebot.log import logger
from nonebot.plugin import PluginMetadata
from nonebot.plugin.load import inherit_supported_adapters

require("nonebot_plugin_uninfo")
require("nonebot_plugin_alconna")
require("nonebot_plugin_apscheduler")
require("nonebot_plugin_localstore")
from nonebot_plugin_apscheduler import scheduler

from .config import Config

__plugin_meta__ = PluginMetadata(
    name="堡垒之夜游戏插件",
    description="堡垒之夜战绩，季卡，商城，vb图查询",
    usage="季卡/生涯季卡/战绩/生涯战绩/商城/vb图",
    type="application",
    config=Config,
    homepage="https://github.com/fllesser/nonebot-plugin-fortnite",
    supported_adapters=inherit_supported_adapters("nonebot_plugin_alconna", "nonebot_plugin_uninfo"),
)

from .pve import screenshot_vb_img, vb_file
from .shop import screenshot_shop_img, shop_file
from .stats import get_level, get_stats_image


@scheduler.scheduled_job(
    "cron",
    id="fortnite",
    hour=8,
    minute=5,
    misfire_grace_time=300,
)
async def _():
    logger.info("开始更新商城/VB图...")
    try:
        await screenshot_shop_img()
        logger.success(f"商城更新成功，文件大小: {shop_file.stat().st_size / 1024 / 1024:.2f} MB")
    except Exception as e:
        logger.warning(f"商城更新失败: {e}")
    try:
        await screenshot_vb_img()
        logger.success(f"vb图更新成功，文件大小: {vb_file.stat().st_size / 1024 / 1024:.2f} MB")
    except Exception as e:
        logger.warning(f"vb图更新失败: {e}")


import re

from arclet.alconna import Alconna, Args, Arparma
from nonebot import on_command
from nonebot.permission import SUPERUSER
from nonebot_plugin_alconna import AlconnaMatcher, Match, on_alconna
from nonebot_plugin_alconna.uniseg import Image, Text, UniMessage
from nonebot_plugin_uninfo import Uninfo

timewindow_prefix = ["生涯", ""]
name_args = Args["name?", str]


battle_pass_alc = on_alconna(Alconna(timewindow_prefix, "季卡", name_args))
stats_alc = on_alconna(Alconna(timewindow_prefix, "战绩", name_args))


@battle_pass_alc.handle()
@stats_alc.handle()
async def _(matcher: AlconnaMatcher, session: Uninfo, name: Match[str]):
    if name.available:
        matcher.set_path_arg("name", name.result)
        return
    # 获取群昵称
    if not session.member or not session.member.nick:
        return
    pattern = r"(?:id:|id\s)(.+)"
    if matched := re.match(pattern, session.member.nick, re.IGNORECASE):
        matcher.set_path_arg("name", matched.group(1))


name_prompt = UniMessage.template(
    "{:At(user, $event.get_user_id())} 请发送游戏名称\n群昵称设置如下可快速查询:\n    id:name\n    ID name"
)


@battle_pass_alc.got_path("name", prompt=name_prompt)
async def _(arp: Arparma, name: str):
    header: str = arp.header_match.result
    receipt = await UniMessage.text(f"正在查询 {name} 的{header}，请稍后...").send()
    level_info = await get_level(name, header)
    await UniMessage(Text(level_info)).send()
    await receipt.recall(delay=1)


@stats_alc.got_path("name", prompt=name_prompt)
async def _(arp: Arparma, name: str):
    header: str = arp.header_match.result
    receipt = await UniMessage.text(f"正在查询 {name} 的{header}，请稍后...").send()
    try:
        res = await get_stats_image(name, header)
    except Exception as e:
        await UniMessage(Text(f"查询失败 | {e}")).finish()
    await UniMessage(Image(path=res)).send()
    await receipt.recall(delay=1)


shop_matcher = on_command("商城", aliases={"商城图"})


@shop_matcher.handle()
async def _():
    if not shop_file.exists():
        logger.info("商城图不存在, 开始更新商城...")
        try:
            await screenshot_shop_img()
            logger.success(f"商城更新成功，文件大小: {shop_file.stat().st_size / 1024 / 1024:.2f} MB")
        except Exception as e:
            logger.warning(f"商城更新失败: {e}")
    await UniMessage(Image(path=shop_file) + Text("可前往 https://www.fortnite.com/item-shop?lang=zh-Hans 购买")).send()


@on_command("更新商城", permission=SUPERUSER).handle()
async def _():
    receipt = await UniMessage.text("正在更新商城，请稍后...").send()
    try:
        file = await screenshot_shop_img()
        await UniMessage(Text("手动更新商城成功") + Image(path=file)).send()
    except Exception as e:
        await UniMessage(Text(f"手动更新商城失败 | {e}")).send()
    finally:
        await receipt.recall(delay=1)


vb_matcher = on_command("vb图", aliases={"VB图"})


@vb_matcher.handle()
async def _():
    if not vb_file.exists():
        logger.info("vb 图不存在, 开始更新vb图...")
        try:
            await screenshot_vb_img()
            logger.success(f"vb图更新成功, 文件大小: {vb_file.stat().st_size / 1024 / 1024:.2f} MB")
        except Exception as e:
            logger.warning(f"vb图更新失败: {e}")
    await UniMessage(Image(path=vb_file)).send()


@on_command("更新vb图", permission=SUPERUSER).handle()
async def _():
    receipt = await UniMessage.text("正在更新vb图，请稍后...").send()
    try:
        file = await screenshot_vb_img()
        await UniMessage(Text("手动更新 VB 图成功") + Image(path=file)).send()
    except Exception as e:
        await UniMessage(Text(f"手动更新 VB 图失败 | {e}")).send()
    finally:
        await receipt.recall(delay=1)



from pathlib import Path
from typing import Any

import aiofiles
from fortnite_api import Client
from fortnite_api.enums import StatsImageType, TimeWindow
from fortnite_api.errors import FortniteAPIException
import httpx
from PIL import Image, ImageDraw, ImageFont

from .config import cache_dir, data_dir, fconfig

API_KEY: str = fconfig.fortnite_api_key


def handle_fortnite_api_exception(e: FortniteAPIException) -> str:
    err_msg = str(e)
    if "public" in err_msg:
        return "战绩未公开"
    elif "exist" in err_msg:
        return "用户不存在"
    elif "match" in err_msg:
        return "该玩家当前赛季没有进行过任何对局"
    elif "timed out" in err_msg:
        return "请求超时, 请稍后再试"
    elif "failed to fetch" in err_msg:
        return "拉取账户信息失败, 稍后再试"
    else:
        return f"未知错误: {err_msg}"


async def get_level(name: str, cmd_header: str) -> str:
    time_window: Any = TimeWindow.LIFETIME if cmd_header.startswith("生涯") else TimeWindow.SEASON
    try:
        async with Client(api_key=API_KEY) as client:
            stats = await client.fetch_br_stats(name=name, time_window=time_window)
    except FortniteAPIException as e:
        return handle_fortnite_api_exception(e)
    bp = stats.battle_pass
    if bp is None:
        return f"未查询到 {stats.user.name} 的季卡等级"
    return f"{stats.user.name}: Lv{bp.level} | {bp.progress}% to Lv{bp.level + 1}"


async def get_stats_image(name: str, cmd_header: str) -> Path:
    time_window: Any = TimeWindow.LIFETIME if cmd_header.startswith("生涯") else TimeWindow.SEASON
    image_type: Any = StatsImageType.ALL
    try:
        async with Client(api_key=API_KEY) as client:
            stats = await client.fetch_br_stats(
                name=name,
                time_window=time_window,
                image=image_type,
            )
    except FortniteAPIException as e:
        raise ValueError(handle_fortnite_api_exception(e))
    if stats.image is None:
        raise ValueError(f"未查询到 {stats.user.name} 的战绩")
    return await get_stats_img_by_url(stats.image.url, stats.user.name)


font_path: Path = data_dir / "SourceHanSansSC-Bold-2.otf"


async def check_font_file() -> bool:
    from nonebot import logger

    if not font_path.exists():
        # 下载 字体 githubraw https://github.com/fllesser/nonebot-plugin-fortnite/blob/master/fonts/SourceHanSansSC-Bold-2.otf
        import aiofiles
        import httpx

        url = (
            "https://raw.githubusercontent.com/fllesser/nonebot-plugin-fortnite/master/fonts/SourceHanSansSC-Bold-2.otf"
        )
        logger.info(f"字体文件不存在，开始从 {url} 下载字体...")
        try:
            async with httpx.AsyncClient(timeout=60) as client:
                response = await client.get(url)
            response.raise_for_status()
            font_data = response.content

            async with aiofiles.open(font_path, "wb") as f:
                await f.write(font_data)

            logger.success(f"字体 {font_path.name} 下载成功，文件大小: {font_path.stat().st_size / 1024 / 1024:.2f} MB")
        except Exception as e:
            logger.error(f"字体下载失败: {e}")
            logger.warning(f"请前往仓库下载字体到 {data_dir}/，否则战绩查询可能无法显示中文名称")
            return False
    return True


async def get_stats_img_by_url(url: str, name: str) -> Path:
    file = cache_dir / f"{name}.png"
    async with httpx.AsyncClient() as client:
        resp = await client.get(url)
    resp.raise_for_status()

    async with aiofiles.open(file, "wb") as f:
        await f.write(resp.content)
    # 如果不包含中文名，返回原图
    if not (await check_font_file()) or not contains_chinese(name):
        return file

    with Image.open(file) as img:
        draw = ImageDraw.Draw(img)

        # 矩形区域的坐标
        left, top, right, bottom = 26, 90, 423, 230

        # 创建渐变色并填充矩形区域
        width = right - left
        height = bottom - top

        start_color = (0, 33, 69, 255)
        end_color = (0, 82, 106, 255)
        for i in range(width):
            for j in range(height):
                r = int(start_color[0] + (end_color[0] - start_color[0]) * (i + j) / (width + height))
                g = int(start_color[1] + (end_color[1] - start_color[1]) * (i + j) / (width + height))
                b = int(start_color[2] + (end_color[2] - start_color[2]) * (i + j) / (width + height))
                draw.point((left + i, top + j), fill=(r, g, b))

        # 指定字体
        font_size = 36
        # hansans = data_dir / "SourceHanSansSC-Bold-2.otf"
        font = ImageFont.truetype(font_path, font_size)

        # 计算字体坐标
        length = draw.textlength(name, font=font)
        x = left + (right - left - length) / 2
        y = top + (bottom - top - font_size) / 2
        draw.text((x, y), name, fill="#fafafa", font=font)

        # 保存
        img.save(file)
        return file


def contains_chinese(text):
    import re

    pattern = re.compile(r"[\u4e00-\u9fff]")
    return bool(pattern.search(text))



from pathlib import Path

from nonebot import get_plugin_config
import nonebot_plugin_localstore as store
from pydantic import BaseModel


class Config(BaseModel):
    fortnite_api_key: str = ""


fconfig: Config = get_plugin_config(Config)

cache_dir: Path = store.get_plugin_cache_dir()
data_dir: Path = store.get_plugin_data_dir()



import asyncio
from pathlib import Path

from nonebot.log import logger
from PIL import Image
from playwright.async_api import Locator, Route, async_playwright

from .config import cache_dir, data_dir

vb_file = data_dir / "vb.png"
hot_info_1_path = cache_dir / "hot_info_1.png"
container_hidden_xs_path = cache_dir / "container_hidden_xs.png"
hot_info_2_path = cache_dir / "hot_info_2.png"


async def screenshot_vb_img() -> Path:
    url = "https://freethevbucks.com/timed-missions"

    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True)
        context = await browser.new_context()

        # 拦截广告
        async def ad_block_handler(route: Route):
            ad_domains = [
                "googlesyndication.com",
                "doubleclick.net",
                "adnxs.com",
                "google-analytics.com",
                "facebook.com",
                "amazon-adsystem.com",
                "adform.net",
                "googleadservices.com",
                "doubleclick.net",
            ]
            if any(ad_domain in route.request.url for ad_domain in ad_domains):
                await route.abort()
            else:
                await route.continue_()

        await context.route("**/*", ad_block_handler)

        page = await context.new_page()
        await page.goto(url)

        # 截图函数，超时则跳过
        async def take_screenshot(locator: Locator, path: Path) -> None:
            try:
                # 检查元素内容是否为空
                content = await locator.inner_html()
                if content.strip():
                    await asyncio.wait_for(locator.screenshot(path=path), timeout=5)
                else:
                    logger.warning(f"Locator for {path.name} is empty.")
            except Exception:
                pass

        # 截取第一个 <div class="hot-info">
        hot_info_1 = page.locator("div.hot-info").nth(0)
        await take_screenshot(hot_info_1, hot_info_1_path)

        # 截取 <div class="container hidden-xs">
        container_hidden_xs = page.locator("div.container.hidden-xs")
        await take_screenshot(container_hidden_xs, container_hidden_xs_path)

        # 截取第二个 <div class="hot-info">
        hot_info_2 = page.locator("div.hot-info").nth(1)
        await take_screenshot(hot_info_2, hot_info_2_path)

    combine_imgs()
    return vb_file


def combine_imgs():
    # 打开截图文件（如果存在）
    combined_image = None
    img_paths = [hot_info_1_path, container_hidden_xs_path, hot_info_2_path]
    img_paths = [i for i in img_paths if i.exists()]
    if not img_paths:
        raise Exception("所有选择器的截图文件均不存在")
    images = []
    try:
        images = [Image.open(img_path) for img_path in img_paths]
        # 获取尺寸并创建新图像
        widths, heights = zip(*(img.size for img in images))
        total_width = max(widths)
        total_height = sum(heights)
        combined_image = Image.new("RGB", (total_width, total_height))

        # 将截图粘贴到新图像中
        y_offset = 0
        for img in images:
            combined_image.paste(img, (0, y_offset))
            y_offset += img.height

        # 保存合并后的图像
        combined_image.save(vb_file)
    finally:
        # 关闭并删除所有截图文件
        for img in images:
            img.close()
        for img_path in img_paths:
            img_path.unlink()
        if combined_image:
            combined_image.close()


async def screenshot_fortnitedb() -> Path:
    url = "https://fortnitedb.com"
    fortnitedb_file = data_dir / "fortnitedb.png"
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True)
        page = await browser.new_page()
        await page.goto(url)
        # 等待加载结束，截图
        await page.wait_for_load_state("networkidle")
        await page.screenshot(path=fortnitedb_file)
        await browser.close()
    return fortnitedb_file



import asyncio
from pathlib import Path

from playwright.async_api import async_playwright

from .config import data_dir

shop_file = data_dir / "shop.png"


async def screenshot_shop_img() -> Path:
    # url = "https://www.fortnite.com/item-shop?lang=zh-Hans"
    url = "https://fortnite.gg/shop"

    headers = {
        "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36",  # noqa: E501
        "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,image/avif,image/webp,image/apng,*/*;q=0.8,application/signed-exchange;v=b3;q=0.9",  # noqa: E501
        "Accept-Encoding": "gzip, deflate",
        "upgrade-insecure-requests": "1",
        "dnt": "1",
        "x-requested-with": "mark.via",
        "sec-fetch-site": "none",
        "sec-fetch-mode": "navigate",
        "sec-fetch-user": "?1",
        "sec-fetch-dest": "document",
        "accept-language": "zh-CN,zh;q=0.9,en-US;q=0.8,en;q=0.7",
        "Cookie": "_sharedid=f02028dd-dce2-4b07-bba9-301d54e68dbd; _sharedid_cst=zix7LPQsHA%3D%3D; _lr_env_src_ats=false; hb_insticator_uid=799b5897-b5a3-48c4-a46f-8bb8bf9082ac",  # noqa: E501
    }
    async with async_playwright() as p:
        browser = await p.chromium.launch(headless=True)
        context = await browser.new_context(extra_http_headers=headers)
        page = await context.new_page()
        # page.on('requestfailed', lambda request: logger.warning(f'Request failed: {request.url}'))
        await page.add_style_tag(content="* { transition: none !important; animation: none !important; }")
        await page.goto(url)

        async def wait_for_load():
            await page.wait_for_load_state("networkidle", timeout=90000)

        async def scroll_page():
            for _ in range(20):
                await page.evaluate("""() => {
                    window.scrollBy(0, document.body.scrollHeight / 20);
                }""")
                await asyncio.sleep(1)  # 等待1秒以加载内容

        await asyncio.gather(wait_for_load(), scroll_page())

        await page.screenshot(path=shop_file, full_page=True)
        return shop_file
